"""
Translates a C file to Kratos2 IR.

Requires pycparser: https://github.com/eliben/pycparser
"""

from __future__ import print_function
import os, sys
import time
from io import StringIO

from c2kratos import options as c_opts
from c2kratos import rewrite as c_rewrite
from c2kratos import ast as c_ast


def getopts(args=None):
    opts = c_opts.Options()
    opts.parse(args)
    return opts


def main(opts):
    #opts = getopts()
    out = sys.stdout if not opts.output_file else open(opts.output_file, 'w')
    try:
        pr = out.write
        ast = c_ast.parse_files(opts)

        if opts.print_ast:
            buf = StringIO()
            #ast.show(buf)
            g = c_ast.c_generator.CGenerator()
            buf.write(g.visit(ast))
            for l in buf.getvalue().splitlines():
                pr('; ')
                pr(l)
                pr('\n')
            pr('\n')

        program = c_ast.get_program(opts, ast)
        if opts.svcomp_spec:
            program = opts.svcomp_spec.add(program)
        program = c_rewrite.rewrite(opts, program)
        pr(';; -*- kratos2 -*-\n')
        pr(';; generated by %s on %s\n\n' %
           (os.path.basename(sys.argv[0]),
            time.strftime('%a, %d %b %Y %H:%M:%S')))
        program.write(out)
    finally:
        if out is not sys.stdout:
            out.close()
